@@ -4,14 +4,15 @@ import os
 import difflib
 import re
 import codecs
+import time
 
 from web3 import Web3
+from eth_account import Account
 import requests
 
 from peth.eth.sigs import ERC20Signatures, Signature
 from peth.eth.utils import selector_to_sigs, sha3_256, SelectorDatabase, convert_value, hex2bytes, CoinPrice
 from peth.eth.bytecode import Code
-from peth.eth.opcodes import OpCode
 from peth.core.peth import Peth
 from peth.util import diff
 from peth.util.graph import ContractRelationGraph
@@ -59,6 +60,9 @@ class PethConsole(cmd.Cmd):
             print("Error: ", e)
             if self._debug:
                 raise Exception from e
+            
+            cmd = line.split()[0]
+            super().onecmd(f"help {cmd}")
             return False  # don't stop
 
     def start_console(self):
@@ -184,8 +188,10 @@ class PethConsole(cmd.Cmd):
             value = int(arg)
         print("Value: %s" % value)
         print("Value: %e" % (value/1.0))
-        print("Value/1e6: %s" % (value/1e6))
-        print("Value/1e18: %s" % (value/1e18))
+        print("Value/1e6  (USDT): %s" % (value/1e6))
+        print("Value/1e9  (GWei): %s" % (value/1e9))
+        print("Value/1e18 (Ether): %s" % (value/1e18))
+        print("Hex: %#x" % value)
         print("Hex(Address): %#0.40x" % value)
         print("Hex(Uint256): %0.64x" % value)
 
@@ -323,6 +329,50 @@ class PethConsole(cmd.Cmd):
             print("= %0.1f hours" % (ts/3600))
             print("= %0.1f days" % (ts/3600/24))
 
+    def do_aes(self, arg):
+        """
+        aes enc <plain text> <password>: Encrypt with AES.
+        aes dec <hex secret> <password>: Decrypt with AES. 
+        
+        NOTE: IV used as MD5 of password, which is not so safe.
+        """
+        args = arg.split()
+
+        # https://www.pycryptodome.org
+        from Crypto.Cipher import AES
+        from Crypto.Util.Padding import pad, unpad
+        import hashlib
+
+        txt = args[1]
+        password = args[2]
+        
+        key = pad(bytes(password, 'utf-8'), 16)
+        iv = hashlib.md5(bytes(password, 'utf-8')).digest()
+
+        def do_enc(plain):
+            plain = pad(bytes(plain, 'utf-8'), 16)
+            aes = AES.new(key, AES.MODE_CBC, iv=iv)
+            secret = aes.encrypt(plain)
+            hex_secret = secret.hex()
+            return hex_secret
+        
+        def do_dec(hex_secret):
+            aes = AES.new(key, AES.MODE_CBC, iv=iv)
+            secret = bytes.fromhex(hex_secret)
+            plain = unpad(aes.decrypt(secret), 16)
+            return plain.decode("utf-8")
+
+        if args[0] == "enc":
+            print(do_enc(txt))
+        else:
+            assert args[0] == "dec", "args[0] is not enc/dec"
+            try:
+                plain = do_dec(txt)
+                print(plain)
+            except Exception as e:
+                print("[!] %s (Password may be wrong)" % e)
+
+
     ##################################################################
     # Basic ETH commands.
 
@@ -435,6 +485,75 @@ class PethConsole(cmd.Cmd):
         addr = self.web3.toChecksumAddress(arg)
         print("Size", len(self.web3.eth.get_code(addr)))
 
+    def do_signer(self, arg):
+        """
+        signer <private-key>: Set signer for send_tx. DANGEROUS!
+        """
+        if self.peth.signer:
+            print("Old:", self.peth.signer.address)
+        else:
+            print("Old: Not set.")
+
+        if not arg:
+            return
+
+        signer = Account.from_key(arg)
+        self.peth.signer = signer
+        print("New:", self.peth.signer.address)
+
+    def do_send_tx(self, arg):
+        """
+        send_tx <data> [<to>] [<value>] : Send tx. DANGEROUS!
+        """
+        assert self.peth.signer, "Use `signer` to set your key."
+        args = arg.split()
+        data = args[0]
+        to = None
+        value = None
+        if len(args) >= 2:
+            to = args[1]
+        if len(args) >= 3:
+            value = args[2]
+            
+        tx, signed_tx = self.peth.send_transaction(data, to, value, True)
+        print("TX info:")
+        self.__print_json(tx, True)
+        print("Sig info:")
+        tx_hash = signed_tx.hash.hex()
+        print("  Hash:\t", tx_hash)
+        print("  Raw Transaction:\t", signed_tx.rawTransaction.hex())
+        print("  r:\t", signed_tx.r)
+        print("  s:\t", signed_tx.s)
+        print("  v:\t", signed_tx.v)
+        print("RPC:", self.peth.rpc_url)
+
+        if input("Enter YES to send:") != "YES":
+            print("Cancelled.")
+            return
+
+        current_block = self.peth.web3.eth.block_number
+        self.peth.web3.eth.send_raw_transaction(signed_tx.rawTransaction)
+        print(f"Sent {tx_hash}. Current block {current_block}")
+        while True:
+            new_block = self.peth.web3.eth.block_number
+            if new_block == current_block:
+                time.sleep(1)
+                continue
+            
+            print(f"New block {new_block}")
+            current_block = new_block
+            try:
+                rcpt = self.web3.eth.get_transaction_receipt(tx_hash)
+                try:
+                    self.do_tx(tx_hash)
+                except:
+                    pass
+                print("Full Receipt:")
+                self.__print_json(rcpt, True)
+                break
+            except Exception as e:
+                print(e)
+            
     ##################################################################
     # Common used eth_call alias command.
 
@@ -761,14 +880,47 @@ class PethConsole(cmd.Cmd):
             sender = info["from"]
             to = info["to"]
             data = info["input"]
+            r = self.web3.eth.get_transaction_receipt(txid)
             if to is None:
-                r = self.web3.eth.get_transaction_receipt(txid)
                 contract_address = r["contractAddress"]
                 print("%s creates contract %s" % (sender, contract_address))
+            else:
+                print("%s -> %s" % (sender, to))
+                if data:
+                    self.peth.decode_call(to, data)
+    
+            if r.status == 0:
+                print("Reverted.")
                 return
-            print("%s -> %s" % (sender, to))
-            if data:
-                self.peth.decode_call(to, data)
+
+            erc20_trans = []
+            for item in r.logs:
+                address_names = {
+                    sender: "sender",
+                    to: "to"
+                }
+
+                # Transfer event.
+                if len(item.topics) == 3 and item.topics[0].hex() == "0xddf252ad1be2c89b69c2b068fc378daa952ba7f163c4a11628f55a4df523b3ef":
+                    src = self.web3.toChecksumAddress(item.topics[1][-20:])
+                    src = address_names.get(src, src)
+                    dst = self.web3.toChecksumAddress(item.topics[2][-20:])
+                    dst = address_names.get(dst, dst)
+
+                    amount = self.web3.toInt(hexstr = item.data)
+                    token = item.address
+                    symbol = self.peth.call_contract(token, "symbol()->(string)", silent=True)
+                    if symbol is None:
+                        symbol = "Unknown"
+                    token = '%s(%s)' % (symbol, token)
+                    msg = ' '.join(map(str, (token, '%s->%s' %(src, dst), amount)))
+                    erc20_trans.append(msg)
+            
+            if erc20_trans:
+                print("ERC20 Transfers:")
+                for msg in erc20_trans:
+                    print(" ", msg)
+
         else:
             assert len(args) == 2, "Invalid args number."
             sig_or_addr = args[0]
@@ -1186,7 +1338,8 @@ class PethConsole(cmd.Cmd):
     def do_url(self, arg):
         """
         url <addr> : Open blockchain explorer of the address.
-        url <tx> : Open blockchain explorer of the tx.
+        url <tx> : Open tx.eth.samczsun.com of the tx.
+        url <tx>& : Add a "&" to open blockchain explorer of the tx.
         """
         sam_chains = {
             "eth": "ethereum",
@@ -1263,8 +1416,9 @@ class PethConsole(cmd.Cmd):
                 raw_balance = token["balance"]
                 balance = token["balance"]/(10**decimals)
 
-                print(f"\t%-5s %-30s\t%-10.2f\t$ %-10.2f\t%-10s\t%-20s" %
-                    (symbol, name, balance, balance*price, raw_balance,price))
+                if balance*price > 1: # Only print asset more than $1.0
+                    print(f"\t%-5s %-30s\t%-10.2f\t$ %-10.2f\t%-10s\t%-20s" %
+                        (symbol, name, balance, balance*price, raw_balance,price))
 
     def do_price(self, arg):
         """
@@ -1278,11 +1432,12 @@ class PethConsole(cmd.Cmd):
             tokens = coin.get_token(chain, *args)
             for token in tokens:
                 if token:
-                    print("%s %s %s %s" % (
+                    print("%s %s decimals %s price %s confidence %s" % (
                         token["symbol"],
                         token["address"],
                         token["decimals"],
-                        token["price"]
+                        token["price"],
+                        token["confidence"]
                     ))
                 else:
                     print("Unknown")
